private import SDL;
private import SDL_image;
private import tango.stdc.stringz;
private import tango.text.convert.Format;

abstract class AbstractImage {
  private const SDL_Surface* _surface;
  protected const uint _width;
  protected const uint _height;

  this(SDL_Surface* surface) {
    this._surface = surface;
    this._width = surface.w;
    this._height = surface.h;
  }

  protected SDL_Surface* surface() {
    return _surface;
  }

  uint width() {
    return _width;
  }

  uint height() {
    return _height;
  }

  void drawImage(Image image) {
    drawImage(image, 0, 0, image.width, image.height, 0, 0);
  }

  void drawImage(Image image, int srcX, int srcY, uint srcW, uint srcH, int x, int y) {
    auto srcRect = SDL_Rect(cast(short)srcX, cast(short)srcY, cast(ushort)srcW, cast(ushort)srcH);
    auto rect = SDL_Rect(cast(short)x, cast(short)y, cast(ushort)srcW, cast(ushort)srcH);
    SDL_BlitSurface(image.surface, &srcRect, surface, &rect);
  }
}

final class Image : AbstractImage {
  this(char[] fileName) {
    SDL_Surface* surface = IMG_Load(toStringz(fileName));
    if (surface is null) {
      throw new Exception(Format("failed to load an image from {}.", fileName));
    }
    super(surface);
  }

  ~this() {
    SDL_FreeSurface(surface);
  }
}

final class Screen : AbstractImage {
  this(uint width, uint height) {
    SDL_Surface* screen = SDL_SetVideoMode(width, height, 32, SDL_DOUBLEBUF);
    if (screen is null) {
      throw new Exception(Format("failed to set the video mode to (w={}, h={}).", width, height));
    }
    super(screen);
  }

  bool flip() {
    return SDL_Flip(surface) == 0;
  }

  /// ウィンドウのキャプションを変更します
  void caption(char[] caption) {
    SDL_WM_SetCaption(toStringz(caption), null);
  }

  void clear() {
    SDL_FillRect(surface, null, SDL_MapRGB(surface.format, 0xFF, 0xFF, 0xFF));
  }
}