final class Mouse {
  private int x;
  private int xMin;
  private int xMax;

  private uint _swingCount;
  
  // マウスを左右に振ったと判定される最小のピクセル幅
  private const int threshold;
  
  private enum SwingState {
    UNREGISTERED,
    NEUTRAL,
    LEFT,
    RIGHT,
  }

  private SwingState state;

  this(int threshold) {
    this.threshold = threshold;
    reset;
  }

  private void reset() {
    _swingCount = 0;
    state = SwingState.UNREGISTERED;
  }

  void click() {
    reset;
  }

  void move(int x, int y) {
    switch (state) {
    case SwingState.UNREGISTERED:
      this.x = this.xMin = this.xMax = x;
      state = SwingState.NEUTRAL;
      break;
    case SwingState.NEUTRAL:
      this.xMax = max(this.xMax, x);
      this.xMin = min(this.xMin, x);
      if (xMax - this.x > threshold) {
        state = SwingState.RIGHT;
        this.x = x;
        this.xMin = x;
      } else if (this.x - xMin > threshold) {
        state = SwingState.LEFT;
        this.x = x;
        this.xMax = x;
      }
      break;
    case SwingState.LEFT:
      if (x - xMin > threshold) {
        state = SwingState.RIGHT;
        this.x = x;
        this.xMax = x;
        _swingCount++;
      }
      break;
    case SwingState.RIGHT:
      if (xMax - x > threshold) {
        state = SwingState.LEFT;
        this.x = x;
        this.xMin = x;
        _swingCount++;
      }
      break;
    default:
    }
  }

  uint swingCount() {
    return _swingCount;
  }

  private int min(int a, int b) {
    return a > b ? b : a;
  }

  private int max(int a, int b) {
    return a > b ? a : b;
  }
}